////////////////////////////////////////////////////////////////////////////////
///
/// @file       projects/custom_serial/custom_serial.c
///
/// @project    EM9304 SOC SDK Example
///
/// @brief      Example Bluetooth Low Energy example application demonstrating
///             the CustomProfileServer profile to create a custom serial port
///             profile.  The custom profile provides a transmit and receive
///             characteristic that the remote device can read and write after
///             completing the connection.  The data received is then sent to
///             the UART peripheral.  Data from the UART peripheral is written
///             to the transmit characteristic.
///
///             To create a new application based on this example, follow these
///             steps:
///             1. Create a new application folder under the SDK "projects"
///                folder.
///             2. Copy the example application files to the new application
///                folder.  Only the source files (*.h and *.c) and the
///                CMakelists.txt file need be copied to the new application
///                folder.
///             3. Modify the application specific CMakelists.txt file as
///                instructed in the comment sections of the file.
///             4. Add the application folder to the projects CMakelists.txt
///                located in the [SOC SDK path]/sw/projects folder.
///             5. In the Metaware IDE, press F5 to refresh the files.
///             6. In the Metaware IDE, select Project->Build All.
///
////////////////////////////////////////////////////////////////////////////////
///
/// @copyright Copyright (C) 2015-present EM Microelectronic-US Inc.
/// @cond
///
/// All rights reserved.
///
/// IMPORTANT - PLEASE CAREFULLY READ THE FOLLOWING SOURCE CODE LICENSE
/// AGREEMENT, WHICH IS LEGALLY BINDING.  THE SOURCE CODE MAY BE USED ONLY IF
/// YOU ACCEPT THE TERMS AND CONDITIONS OF THIS SOURCE CODE LICENSE AGREEMENT
/// (hereafter, the Agreement).
///
/// By Using this source code, you: (i) warrant and represent that you have
/// obtained all authorizations and other applicable consents required
/// empowering you toenter into and (ii) agree to be bound by the terms of this
/// Agreement on your own behalf and/or on behalf of your company.  If you do
/// not agree to this Agreement, then you are not permitted to Use this source
/// code, in whole or in part.
///
/// License Grant.  Pursuant to the terms in the accompanying software license
/// agreement and Terms of Use located at:
/// https://forums.emdeveloper.com/emassets/emus_sourcelicense.html and
/// https://forums.emdeveloper.com/emassets/emus_termsofuse.html  (the terms of
/// each are incorporated herein by this reference) and subject to the
/// restrictions, disclaimer and limitation of liability set forth below,
/// EM Microelectronic US Inc. (EM), grants strictly to you, without the right
/// to sublicense, a non-exclusive, non-transferable, revocable, limited
/// worldwide license to use the source code to modify or create derivative
/// works of the software program for the sole purpose of developing object
/// and executable versions that execute solely and exclusively on devices
/// manufactured by or for EM or your products that use or incorporate devices
/// manufactured by or for EM.
///
/// Restriction.  Without limiting any of the foregoing, the name
/// "EM Microelectronic-US Inc." or that of any of the EM Parties (as such term
/// is defined below) must not be Used (as such term is defined below) to
/// endorse or promote products derived from the source code without prior
/// written permission from an authorized representative of EM Microelectronic
/// US Inc. Unless authorized officers of the parties to this Agreement have
/// duly executed a special written contract specifically governing such Use,
/// in no event shall the source code be Used in or with: (i) life-critical
/// medical equipment, products or services, (ii) military, armament, nuclear or
/// aerospace applications or environments; or (iii) automotive applications
/// unless specifically designated by EM as automotive-grade.  In addition, you
/// understand and agree that you remain solely responsible for using your
/// independent analysis, evaluation, and judgment in designing and developing
/// your applications, products and services.
///
/// DISCLAIMER.  THE SOURCE CODE IS PROVIDED "AS IS" WITH ALL FAULTS, WITHOUT
/// ANY SUPPORT. EM MAKES NO WARRANTIES OR REPRESENTATIONS, EXPRESS, IMPLIED OR
/// STATUTORY, INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
/// A PARTICULAR PURPOSE OR USE, LACK OF VIRUSES, ACCURACY OR COMPLETENESS OF
/// RESPONSES, RESULTS, LACK OF NEGLIGENCE AND LACK OF SECURITY.  EM DISCLAIMS
/// ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET POSSESSION, AND NON-
/// INFRINGEMENT OF ANY THIRD PARTY INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO
/// THE SOURCE CODE OR YOUR USE OF THE SOURCE CODE.  THE ENTIRE RISK AS TO THE
/// QUALITY AND PERFORMANCE OF THE SOURCE CODE IS WITH YOU. SHOULD THE SOURCE
/// CODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR
/// OR CORRECTION.  NOTHING CONTAINED IN THIS AGREEMENT WILL BE CONSTRUED AS A
/// WARRANTY OR REPRESENTATION BY ANY EM PARTIES TO MAINTAIN PRODUCTION OF ANY
/// EM DEVICE OR OTHER HARDWARE OR SOFTWARE WITH WHICH THE SOURCE CODE MAY BE
/// USED.  NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY EM OR AN EM
/// AUTHORIZED REPRESENTATIVE WILL CREATE A WARRANTY
///
/// LIMITATION OF LIABILITY.  IN NO EVENT SHALL EM MICROELECTRONIC US INC., ITS
/// PARENT AND ITS AND THEIR RESPECTIVE AFFILIATES, SUBSIDIARIES, LICENSORS,
/// THIRD PARTY PROVIDERS, REPRESENTATIVES, AGENTS AND ASSIGNS (COLLECTIVLEY,
/// EM PARTIES) BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
/// PUNITIVE, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
/// EQUIPMENT, SYSTEMS, SOFTWARE, TECHNOLOGY, SERVICES, GOODS, CONTENT,
/// MATERIALS OR PROFITS; BUSINESS INTERRUPTION OR OTHER ECONOMIC LOSS OR ANY
/// CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF)
/// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
/// OUT OF THE USE OF THIS SOURCE CODE, INCLUDING ANY PORTION(S) THEREOF, EVEN
/// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NOTWITHSTANDING ANYTHING ELSE
/// TO THE CONTRARY, IN NO EVENT WILL THE EM PARTIES AGGREGATE LIABILITY UNDER
/// THIS AGREEMENT OR ARISING OUT OF YOUR USE OF THE SOURCE CODE EXCEED ONE
/// HUNDRED U.S. DOLLARS (U.S. $100).
///
/// Because some jurisdictions do not allow the exclusion or limitation of
/// incidental or consequential damages or limitation on how long an implied
/// warranty lasts, the above limitations or exclusions may not apply to you.
///
/// Please refer to the accompanying software license agreement and Terms of Use
/// located at, respectively:
/// https://forums.emdeveloper.com/emassets/emus_sourcelicense.html and
/// https://forums.emdeveloper.com/emassets/emus_termsofuse.html  to better
/// understand all of your rights and obligations hereunder.  Except as
/// otherwise provided in this Agreement, all capitalized terms defined in said
/// software license agreement and Terms of Use shall have the meanings
/// ascribed to such terms therein.
/// @endcond
////////////////////////////////////////////////////////////////////////////////

//
// Include Files
//
#include <types.h>
#include <macros.h>

#include <em_qpn.h>
#include <bsp.h>
#include <gpio.h>
#include <platform.h>
#include <pml.h>
#include <spi_slave.h>
#include <t9304_periph.h>
#include <uart.h>

// Standard Includes
#include <string.h>

// BLE Includes
#include <BleBase.h>
#include <BleGap.h>
#include <BleEngine.h>
#include <AciCoreCallback.h>  // Enable vendor specific ACI commands

// Application Includes
#include "uart_ble_custom_serial.h"
#include "uart_ble_custom_serial_jli.h"


//
//  Constant Definitions
//
#define GPIO_CUSTOMSERIAL_ALERT_LED ( 2u )

// GPIO to use for the UART.
#define UART_RX_GPIO (0u)
#define UART_TX_GPIO (1u)
// GPIO to use for status led
#define LINK_GPIO (2u)
#define ACTIVITY_GPIO (3u)

#define BLEINFOTYPE_SAVED_CLIENTCONFIG ( 0xFF )
#define CUSTOMSERIAL_MAX_VALUE_SIZE    (  16u )
#define CUSTOMSERIAL_RX_COUNT_TO_GET   (   1u )
#define CUSTOMSERIAL_ENABLE_ECHO
#define EXAMPLE_LOCAL_DEVICE_NAME ("Uart Ble")

//
//  Type Definitions
//
typedef struct
{
    // Number of bytes in the buffer.
    U16 count;

    // Buffer.
    U8 data[ CUSTOMSERIAL_MAX_VALUE_SIZE ];
} DataBuffer_t;


//
//  Global Variables
//
// This variable represents the task.
MyTask_t gMyTask;

// UART buffers
DataBuffer_t gRxBuffer;
DataBuffer_t gTxBuffer;

//The connection handle
U16 gCurrentConnHandle;

// The main service memory
AttServiceAttribute serialService;

//The characteristic memory
Att128BitCharacteristicAttribute writeCharacteristicValue;

//The characteristic memory
Att128BitCharacteristicAttribute readCharacteristicValue;

// The Attribute Value memory
AttAttribute writeAttribute;

// The Attribute Value memory
AttAttribute readAttribute;

//the CCCD
AttInstantiatedAttribute serialClientConfigAttribute;

U8 serialClientConfigMemory[ 2 * BLE_NUM_MAX_CONNECTION ];

// The Attribute Value
U8 writeValue[ CUSTOMSERIAL_MAX_VALUE_SIZE ];

// The Attribute Value
U8 readValue[ CUSTOMSERIAL_MAX_VALUE_SIZE ];

// custom service UUID
// 128 bit UUID is 0x6e400001-b5a3-f393-e0a9-e50e24dcca9e
CONST_DECL U8 serialServiceUUIDValue[] = { 0x9e, 0xca, 0xdc, 0x24, 0x0e, 0xe5,
        0xa9, 0xe0, 0x93, 0xf3, 0xa3, 0xb5, 0x01, 0x00, 0x40, 0x6e };

// Read characteristic UUID
// 128 bit UUID is 0x6e400003-b5a3-f393-e0a9-e50e24dcca9e
CONST_DECL U8 writeUUIDValue[] = { 0x9e, 0xca, 0xdc, 0x24, 0x0e, 0xe5,
        0xa9, 0xe0, 0x93, 0xf3, 0xa3, 0xb5, 0x03, 0x00, 0x40, 0x6e };

// Write characteristic UUID
// 128 bit UUID is 0x6e400002-b5a3-f393-e0a9-e50e24dcca9e
CONST_DECL U8 readUUIDValue[] = { 0x9e, 0xca, 0xdc, 0x24, 0x0e, 0xe5,
        0xa9, 0xe0, 0x93, 0xf3, 0xa3, 0xb5, 0x02, 0x00, 0x40, 0x6e };


//
//  Function Prototypes
//

/**
 * @brief Called during a stack event.
 */
void CustomSerial_CoreCallback(BleEvent event, BleStatus status, void* pParam);

/**
 * @brief Called during a GAP event.
 */
void CustomSerial_GapCallback(BleGapEvent event, BleStatus status, void *pParms);

/**
 * @brief Called when a UART character is received.
 */
void CustomSerial_UartCallback(Driver_Status_t status, void *pUserData);

/**
 * @brief Register a specific Service.
 *
 * This function is used to register and initialize the custom
 * service.  The specific service and characteristic will be added to
 * attribute database.
 *
 * @return The status of the operation:
 *  - BLESTATUS_SUCCESS indicates that the operation succeeded.
 *  - BLESTATUS_FAILED indicates that the operation has failed, merely
 *    because the specific Service is already registered.
 *
 */
BleStatus CustomSerial_Register( void );

/**
 * @brief Handle the specific Service Attribute callback.
 *
 * This function is used to catch the specific Service Attribute
 * callback.
 *
 * @param serviceCallbackParms  Service callback parameters
 *
 * @return none
 */
static void CustomSerial_AttServiceCallback(
        AttServerCallbackParms *serverCallbackParms);

/**
 * @brief Waiting for Bluetooth Low Energy stack initialization.
 *
 * @param me Pointer to the task.
 * @returns Next action for the state machine to take (stay or transition).
 */
QState CustomSerial_WaitForStack(MyTask_t *me);

/**
 * @brief Main state of the customer serial application.
 *
 * @param me Pointer to the task.
 * @returns Next action for the state machine to take (stay or transition).
 */
QState CustomSerial_Run(MyTask_t *me);

/**
 * @brief Where programs go to die.
 *
 * @param me Pointer to the task.
 * @returns Next action for the state machine to take (stay or transition).
 */
QState CustomSerial_Error(MyTask_t *me);


/**
 * @brief Initialize the UART.
 */
void CustomSerial_InitUart(void);


////////////////////////////////////////////////////////////////////////////////
//
//  Callback functions
//
////////////////////////////////////////////////////////////////////////////////

void CustomSerial_CoreCallback(BleEvent event, BleStatus status, void *pParam)
{
    switch(event)
    {
        case BLEEVENT_INITIALIZATION_RSP:
            // Notify the task.
            QACTIVE_POST(&gMyTask.super, (int)CUSTOMSERIAL_BLE_INIT_SIG,
                (QParam)status);
            break;

#ifndef EXAMPLE_DISABLE_SPI_SLAVE

        // Handle the vendor specific ACI commands to allow the Configuration
        // Editor to communicate with the EM9304 DVK with the application
        // active.  This allows you to invalidate OTP patches with the
        // Configuration Editor.  When the BLE stack is active, ACI commands
        // are ignored unless the vendor specific events are handled here.
        // After development is complete, this functionality can be removed
        // if no longer needed.
        case BLEEVENT_VENDOR_EVENT:
        case BLEEVENT_VENDOR_COMMAND_STATUS:
        case BLEEVENT_VENDOR_COMMAND_COMPLETE:
            BleAci_CoreCallback(event, status, pParam);
            break;

#endif  // EXAMPLE_DISABLE_SPI_SLAVE

        default:
            break;
    }
}


void CustomSerial_GapCallback(BleGapEvent event, BleStatus status, void *pParam)
{
    BleGapConnectionInformation *pConnectionInfo;

    switch ( event )
    {
        case BLEGAP_EVENT_CONNECTED:
            // Handle connected event.  The stack will handle all aspects of the connection
            // process.
            pConnectionInfo = (BleGapConnectionInformation *) pParam;
            gCurrentConnHandle = pConnectionInfo->connHandle;
            GPIO_SetHigh(LINK_GPIO);
            break;

        case BLEGAP_EVENT_DISCONNECTED:
            (void)BLEGAP_SetMode(BLEMODE_DISCOVERABLE | BLEMODE_CONNECTABLE);
            GPIO_SetLow(LINK_GPIO);
            break;

        default:
            break;
    }
}


void CustomSerial_UartCallback(Driver_Status_t status, void *pUserData)
{
    ( void ) status;
    ( void ) pUserData;

#ifdef CUSTOMSERIAL_ENABLE_ECHO
    // Echo the character and prepare to read more.
    memcpy( &gTxBuffer, &gRxBuffer, sizeof (gTxBuffer ));

    ( void ) UART_SendData(&gTxBuffer.data[ 0 ], gTxBuffer.count);
#endif

    gRxBuffer.count = CUSTOMSERIAL_RX_COUNT_TO_GET; // Receive one byte at a time.
    ( void ) UART_ReceiveDataWithCallback(&gRxBuffer.data[ 0 ], gRxBuffer.count, CustomSerial_UartCallback, NULL);

    //
    // When posting QP-nano events, it is important to use the QK_ISR_ENTRY,
    // QACTIVE_POST_ISR, and QK_ISR_EXIT macros when posting an event
    // from an ISR or non-BLE callback such as a UART, SPI, or universal
    // timer callback.  BLE callbacks would include the CoreCallback,
    // GapCallback, BLE service and profile callbacks, and the FOTA
    // callback.  Non-BLE callbacks such as the universal timer callback
    // require the use the ISR post functions because those callbacks
    // are executed from an ISR whereas the BLE callbacks are not.
    QK_ISR_ENTRY();
    QACTIVE_POST_ISR(&gMyTask.super, ( int ) CUSTOMSERIAL_SEND_INDICATION_SIG, 0u);
    QK_ISR_EXIT();
}


void CustomSerial_AttServiceCallback( AttServerCallbackParms *serverCallbackParms )
{
    U8 *pValue;
    U16 length;

    switch (serverCallbackParms->event)
    {
        //Write request event
        case ATTEVT_SERVER_WRITE_REQ:
            // Write is requested.
            break;

        // Write was completed.
        case ATTEVT_SERVER_WRITE_COMPLETE:
            // If the attribute that get the "write complete event" is customAttribute
            if (serverCallbackParms->parms.writeComplete.attribute  == (const AttAttribute*) &writeAttribute )
            {
                // Post an event to send the TX data.
                QACTIVE_POST(&gMyTask.super, ( int ) CUSTOMSERIAL_TX_SIG, 0u);

                ATT_SERVER_SecureDatabaseAccess();

                // Copy the value from the attribute to the TX buffer.
                if ( ATTSTATUS_SUCCESS == ATT_SERVER_ReadAttributeValue(
                        ( const AttAttribute* ) &writeAttribute,
                        &pValue,
                        &length ))
                {
                    memcpy( &gTxBuffer.data[ 0 ], pValue, length );
                    gTxBuffer.count = length;
                }
                else
                {
                    gTxBuffer.count = 0;
                }
                ATT_SERVER_ReleaseDatabaseAccess();
            }
            break;

        // Read request event.
        case ATTEVT_SERVER_READ_REQ:
            break;

        // Handle value indication event sent.
        case ATTEVT_SERVER_HVI_SENT:
            break;

        //Handle value confirmation event.
        case ATTEVT_SERVER_HANDLE_VALUE_CONFIRMATION:
            break;
    }
}


////////////////////////////////////////////////////////////////////////////////
//
//  Application Specific functions
//
////////////////////////////////////////////////////////////////////////////////

void CustomSerial_InitUart( void )
{
    GPIO_SetInputFunctionPin(GPIO_PIN_FUNC_IN_UART_RXD, UART_RX_GPIO);
    GPIO_DisableOutput(UART_RX_GPIO);
    GPIO_EnableInput(UART_RX_GPIO);

    GPIO_SetOutputPinFunction(UART_TX_GPIO, GPIO_PIN_FUNC_OUT_UART_TXD);
    GPIO_EnableOutput(UART_TX_GPIO);
    GPIO_DisableInput(UART_TX_GPIO);

    // Initialize the receive and transmit buffers.
    ( void ) memset( &gTxBuffer, 0u, sizeof( gTxBuffer ));
    ( void ) memset( &gRxBuffer, 0u, sizeof( gRxBuffer ));

#ifdef CUSTOMSERIAL_ENABLE_ECHO
    gTxBuffer.data[ 0 ] = ( U8 ) '-';
    gTxBuffer.count = 1;

    if ( 0 < gTxBuffer.count )
    {
        ( void ) UART_SendData(&gTxBuffer.data[ 0 ], gTxBuffer.count);
    }
#endif

    gRxBuffer.count = CUSTOMSERIAL_RX_COUNT_TO_GET;
    ( void ) UART_ReceiveDataWithCallback(&gRxBuffer.data[ 0 ], gRxBuffer.count, CustomSerial_UartCallback, NULL);
}


BleStatus CustomSerial_Register( void )
{
    AttUuid typeWrite;
    AttUuid typeRead;

    //Secure the database
    ATT_SERVER_SecureDatabaseAccess();

    // Register The custom service and
    // add the characteristic.
    if (BLESTATUS_FAILED == ATT_SERVER_RegisterServiceAttribute(
            ATTPDU_SIZEOF_128_BIT_UUID,        // UUID length
            (U8 *) serialServiceUUIDValue,     // Service UUID Value
            CustomSerial_AttServiceCallback,   // Service callback
            &serialService ))
    {
        // The service is already registered.
        ATT_SERVER_ReleaseDatabaseAccess();
        return BLESTATUS_FAILED;
    }

    // Add The custom characteristic, which are typically a proprietary 128 bit UUID
    typeWrite.size = ATT_UUID_SIZE_128;
    typeWrite.value.uuid128 = (U8 *) &(writeUUIDValue[0]);

    // Add The custom2 characteristic
    typeRead.size = ATT_UUID_SIZE_128;
    typeRead.value.uuid128 = (U8 *) &(readUUIDValue[0]);

    for (uint8_t i = 0; i < CUSTOMSERIAL_MAX_VALUE_SIZE; i++) {
        writeValue[ i ] = 0x00u;
        readValue[ i ]  = 0x00u;
    }

    //Add characteristic custom Read/Write
    if( BLESTATUS_FAILED == ATT_SERVER_AddCharacteristic(
            (ATTPROPERTY_READ | ATTPROPERTY_WRITE),
            (Att16BitCharacteristicAttribute*) &writeCharacteristicValue,
            &typeWrite,
            ATT_PERMISSIONS_ALLACCESS,
            CUSTOMSERIAL_MAX_VALUE_SIZE,
            ( U8 * ) &(writeValue[ 0 ]),
            0u,
            0u,
            &serialService,
            &writeAttribute ))
    {
        return BLESTATUS_FAILED;
    }

    // Add characteristic custom Read/Write/Notify/Indicate
    if ( BLESTATUS_FAILED == ATT_SERVER_AddCharacteristic(
            ( ATTPROPERTY_READ | ATTPROPERTY_NOTIFY | ATTPROPERTY_INDICATE ),
            (Att16BitCharacteristicAttribute*) &readCharacteristicValue,
            &typeRead,
            ATT_PERMISSIONS_ALLACCESS,
            CUSTOMSERIAL_MAX_VALUE_SIZE,
            ( U8 * ) &(readValue[0]),
            0,
            0,
            &serialService,
            &readAttribute ))
    {
        return BLESTATUS_FAILED;
    }

    // Add characteristic client configuration
    if( ATTSTATUS_FAILED == ATT_SERVER_AddCharacteristicClientConfig(
            ( Att16BitCharacteristicAttribute * ) &readCharacteristicValue,
            (const U8 *) &AttServerClientConfigDefaultValue[0],
            (U8 *) &serialClientConfigMemory[0],
            0, // isValuePersistent
            BLEINFOTYPE_SAVED_CLIENTCONFIG,
            &serialClientConfigAttribute ))
    {
        return BLESTATUS_FAILED;
    }

    //Release the database
    ATT_SERVER_ReleaseDatabaseAccess();

    return BLESTATUS_SUCCESS;
}


void CustomSerial_SendIndication( void ) {

    U8 *readConfigValue;
    U16 readConfigLength;

    //Secure the database
    ATT_SERVER_SecureDatabaseAccess();

    //Write the readAttribute value
    ( void ) ATT_SERVER_WriteAttributeValue(&readAttribute, &gRxBuffer.data[ 0 ], gRxBuffer.count);

    //Read the CCCD attribute value
    if ( BLESTATUS_SUCCESS == ATT_SERVER_ReadInstantiatedAttributeValue(
            &(serialClientConfigAttribute),
            gCurrentConnHandle,
            &readConfigValue,
            &readConfigLength ))
    {
        ( void ) readConfigLength;

        //If the indication is enabled
        if ( 0x0000 != (LE_TO_U16(readConfigValue) & ATT_CLIENTCONFIG_INDICATION ))
        {
            // Send an indication
            ( void ) ATT_SERVER_SendIndication(&readAttribute, gCurrentConnHandle, CustomSerial_AttServiceCallback);
        }

        // If the notification is enabled
        if( 0x0000 != (LE_TO_U16(readConfigValue) & ATT_CLIENTCONFIG_NOTIFICATION ))
        {
            // Send a notification
            ( void ) ATT_SERVER_SendNotification( &readAttribute, gCurrentConnHandle );
        }
    }

    //Release the database
    ATT_SERVER_ReleaseDatabaseAccess();
}


////////////////////////////////////////////////////////////////////////////////
//
//  QP-nano State functions
//
////////////////////////////////////////////////////////////////////////////////

QState CustomSerial_Init(MyTask_t *me)
{
    QState eventState;

#ifdef EXAMPLE_DISABLE_SPI_SLAVE

    // Since the SPI slave peripheral is disabled, ensure the wakeup feature is
    // disabled.  Add any specific wakeup sources at any point after the line.
    PML->RegPmlPadWake.r16[0] = 0; // Disable the wakeup on GPIO0

    for (uint8_t i = 0u; i < 6u; i++)
    {
        // Unused pins must be properly set to avoid significant current draw due
        // to floating pins.
        //
        // For all unsed GPIO, disable the input, output, and pullup function, and
        // enable the pulldown.  If an external pull is applied to the pin, then the
        // internal pullup and pulldown should both be disabled.  In this example,
        // GPIO 0-5 are not used.  For all unsed GPIO, disable the input, output,
        // and pullup functions, and enable the pulldown.  If an external pull is
        // applied to the pin, then the internal pullup and pulldown should both
        // be disabled.  Note that if you disable the GPIO assigned to JTAG, the
        // JTAG port will no longer function.
        GPIO_DisableInput(i);
        GPIO_DisableOutput(i);
        GPIO_SetOutputPinFunction(i, GPIO_PIN_FUNC_OUT_GPIO);

        if (0u == i)
        {
            // For GPIO0, enable the pullup to ensure the GPIO does wakeup
            // the EM9304 if GPIO0 is not used.  This is not a necessary step
            // since the wakeup interrupt is disabled, but added for
            // completeness.
            GPIO_EnablePullUp(i);
            GPIO_DisablePullDown(i);
        }
        else
        {
            GPIO_DisablePullUp(i);
            GPIO_EnablePullDown(i);
        }
    }
#endif    // EXAMPLE_DISABLE_SPI_SLAVE

    // Set the Bluetooth stack core callback.
    me->handler.callback = CustomSerial_CoreCallback;

    // Initialize the Bluetooth stack and register the callback.
    BleStatus bleStatus = BLEMGMT_InitWithHandler(&me->handler);

    if (BLESTATUS_SUCCESS == bleStatus || BLESTATUS_PENDING == bleStatus )
    {
        // Now wait for the stack to finish initialization.
        eventState = Q_TRAN(CustomSerial_WaitForStack);
    }
    else
    {
        // There was an error.
        eventState = Q_TRAN(CustomSerial_Error);
    }

    // GPIO initialization.
    GPIO_DisableInput(LINK_GPIO);
    GPIO_EnableOutput(LINK_GPIO);
    GPIO_SetOutputPinFunction(LINK_GPIO, GPIO_PIN_FUNC_OUT_GPIO );
    GPIO_SetLow(LINK_GPIO);
    GPIO_DisableInput(ACTIVITY_GPIO);
    GPIO_EnableOutput(ACTIVITY_GPIO);
    GPIO_SetOutputPinFunction(ACTIVITY_GPIO, GPIO_PIN_FUNC_OUT_GPIO );
    GPIO_SetLow(ACTIVITY_GPIO);

    return eventState;
}



QState CustomSerial_WaitForStack(MyTask_t *me)
{
    QState eventState;

    switch( Q_SIG(me) )
    {
        case CUSTOMSERIAL_BLE_INIT_SIG:
        {
            if(BLESTATUS_SUCCESS == Q_PAR(me))
            {
                // Initialize the UART.
                CustomSerial_InitUart();

                // Register a GAP device.
                (void)BLEGAP_RegisterDevice(BLEGAPROLE_PERIPHERAL,
                    CustomSerial_GapCallback);

                // Set the name of the peripheral.
                (void)BLEGAP_SetLocalBluetoothDeviceName((U8*) EXAMPLE_LOCAL_DEVICE_NAME,
                    sizeof(EXAMPLE_LOCAL_DEVICE_NAME));

                // Register the custom service.
                (void)CustomSerial_Register();

                // Start advertising.
                (void)BLEGAP_SetMode(BLEMODE_DISCOVERABLE | BLEMODE_CONNECTABLE);

                eventState = Q_TRAN(CustomSerial_Run);
            }
            else
            {
                eventState = Q_TRAN(CustomSerial_Error);
            }

            break;
        }

        default:
            // Let the super class handle this.
            eventState = Q_SUPER(QHsm_top);
            break;
    }

    return eventState;
}


QState CustomSerial_Run(MyTask_t *me)
{
    QState eventState;

    switch( Q_SIG(me) )
    {
        case CUSTOMSERIAL_TX_SIG:
            eventState = Q_HANDLED();
            ( void ) UART_SendData( &gTxBuffer.data[ 0 ], gTxBuffer.count );
            gTxBuffer.count = 0;
            break;

        case CUSTOMSERIAL_SEND_INDICATION_SIG:
            eventState = Q_HANDLED();
            ( void ) CustomSerial_SendIndication();
            break;

        default:
            // Let the super class handle this.
            eventState = Q_SUPER(QHsm_top);
            break;
    }

    return eventState;
}


QState CustomSerial_Error(MyTask_t *me)
{
    QState status;

    switch( Q_SIG(me) )
    {
        default:
            // Let the super class handle this.
            status = Q_SUPER(QHsm_top);
            break;
    }

    return status;
}
